@@ -2,7 +2,9 @@ use git_cliff_core::commit::Commit;
 use git_cliff_core::config::{
 	ChangelogConfig,
 	CommitParser,
+	CommitPreprocessor,
 	GitConfig,
+	LinkParser,
 };
 use git_cliff_core::error::Result;
 use git_cliff_core::release::*;
@@ -15,29 +17,33 @@ use std::fmt::Write;
 fn generate_changelog() -> Result<()> {
 	let changelog_config = ChangelogConfig {
 		header: Some(String::from("this is a changelog")),
-		body:   String::from(
+		body:   Some(String::from(
 			r#"
-        ## Release {{ version }}
-        {% for group, commits in commits | group_by(attribute="group") %}
-        ### {{ group }}    
-        {% for commit in commits %}
-        {%- if commit.scope -%}
-        - *({{commit.scope}})* {{ commit.message }}    
-        {% else -%}
-        - {{ commit.message }}    
-        {% endif -%}
-        {% if commit.breaking -%}
-        {% raw %}  {% endraw %}- **BREAKING**: {{commit.breaking_description}}
-        {% endif -%}
-        {% endfor -%}
+## Release {{ version }}
+{% for group, commits in commits | group_by(attribute="group") %}
+### {{ group }}
+{% for commit in commits %}
+{%- if commit.scope -%}
+- *({{commit.scope}})* {{ commit.message }}{%- if commit.links %} ({% for link in commit.links %}[{{link.text}}]({{link.href}}) {% endfor -%}){% endif %}
+{% else -%}
+- {{ commit.message }}
+{% endif -%}
+{% if commit.breaking -%}
+{% raw %}  {% endraw %}- **BREAKING**: {{commit.breaking_description}}
+{% endif -%}
+{% endfor -%}
 {% endfor %}"#,
-		),
+		)),
 		footer: Some(String::from("eoc - end of changelog")),
 		trim:   None,
 	};
 	let git_config = GitConfig {
-		conventional_commits:  true,
+		conventional_commits:  Some(true),
 		filter_unconventional: Some(true),
+		commit_preprocessors:  Some(vec![CommitPreprocessor {
+			pattern: Regex::new(r#"\(fixes (#[1-9]+)\)"#).unwrap(),
+			replace: String::from("[closes Issue${1}]"),
+		}]),
 		commit_parsers:        Some(vec![
 			CommitParser {
 				message:       Regex::new("^feat").ok(),
@@ -58,8 +64,20 @@ fn generate_changelog() -> Result<()> {
 		tag_pattern:           None,
 		skip_tags:             None,
 		ignore_tags:           None,
-		topo_order:            None,
+		date_order:            None,
 		sort_commits:          None,
+		link_parsers:          Some(vec![
+			LinkParser {
+				pattern: Regex::new("#(\\d+)").unwrap(),
+				href:    String::from("https://github.com/$1"),
+				text:    None,
+			},
+			LinkParser {
+				pattern: Regex::new("https://github.com/(.*)").unwrap(),
+				href:    String::from("https://github.com/$1"),
+				text:    Some(String::from("$1")),
+			},
+		]),
 	};
 
 	let releases = vec![
@@ -74,7 +92,9 @@ fn generate_changelog() -> Result<()> {
 				Commit::new(String::from("abc124"), String::from("feat: add zyx")),
 				Commit::new(
 					String::from("abc124"),
-					String::from("feat(random-scope): add random feature"),
+					String::from(
+						"feat(random-scope): add random feature\n\nThis is related to https://github.com/NixOS/nixpkgs/issues/136814\n\nCloses #123",
+					),
 				),
 				Commit::new(String::from("def789"), String::from("invalid commit")),
 				Commit::new(
@@ -90,6 +110,10 @@ fn generate_changelog() -> Result<()> {
 					String::from("hjkl12"),
 					String::from("chore: do boring stuff"),
 				),
+				Commit::new(
+					String::from("1234"),
+					String::from("fix: support preprocessing (fixes #99)"),
+				),
 			]
 			.iter()
 			.filter_map(|c| c.process(&git_config).ok())
@@ -125,7 +149,7 @@ fn generate_changelog() -> Result<()> {
 	];
 
 	let out = &mut String::new();
-	let template = Template::new(changelog_config.body)?;
+	let template = Template::new(changelog_config.body.unwrap())?;
 
 	writeln!(out, "{}", changelog_config.header.unwrap()).unwrap();
 	for release in releases {
@@ -134,32 +158,34 @@ fn generate_changelog() -> Result<()> {
 	writeln!(out, "{}", changelog_config.footer.unwrap()).unwrap();
 
 	assert_eq!(
-		"this is a changelog
+		r#"this is a changelog
 
-        ## Release v2.0.0
-        
-        ### fix bugs    
-        - fix abc    
-        
-        ### shiny features    
-        - add xyz    
-        - add zyx    
-        - *(random-scope)* add random feature    
-        - *(big-feature)* this is a breaking change    
-          - **BREAKING**: this is a breaking change
-        
-        ## Release v1.0.0
-        
-        ### chore    
-        - do nothing    
-        
-        ### feat    
-        - add cool features    
-        
-        ### fix    
-        - fix stuff    
-        - fix more stuff    
-        eoc - end of changelog\n",
+## Release v2.0.0
+
+### fix bugs
+- fix abc
+- support preprocessing [closes Issue#99]
+
+### shiny features
+- add xyz
+- add zyx
+- *(random-scope)* add random feature ([#123](https://github.com/123) [NixOS/nixpkgs/issues/136814](https://github.com/NixOS/nixpkgs/issues/136814) )
+- *(big-feature)* this is a breaking change
+  - **BREAKING**: this is a breaking change
+
+## Release v1.0.0
+
+### chore
+- do nothing
+
+### feat
+- add cool features
+
+### fix
+- fix stuff
+- fix more stuff
+eoc - end of changelog
+"#,
 		out
 	);
 
