@@ -1,7 +1,10 @@
 use git_cliff_core::commit::Commit;
 use git_cliff_core::config::Config;
 use git_cliff_core::error::Result;
-use git_cliff_core::release::Release;
+use git_cliff_core::release::{
+	Release,
+	Releases,
+};
 use git_cliff_core::template::Template;
 use std::io::Write;
 
@@ -16,19 +19,22 @@ pub struct Changelog<'a> {
 impl<'a> Changelog<'a> {
 	/// Constructs a new instance.
 	pub fn new(releases: Vec<Release<'a>>, config: &'a Config) -> Result<Self> {
+		let mut template = config
+			.changelog
+			.body
+			.as_deref()
+			.unwrap_or_default()
+			.to_string();
+		if config.changelog.trim.unwrap_or(true) {
+			template = template
+				.lines()
+				.map(|v| v.trim())
+				.collect::<Vec<&str>>()
+				.join("\n")
+		}
 		let mut changelog = Self {
 			releases,
-			template: Template::new({
-				let mut template = config.changelog.body.to_string();
-				if config.changelog.trim == Some(true) {
-					template = template
-						.lines()
-						.map(|v| v.trim())
-						.collect::<Vec<&str>>()
-						.join("\n")
-				}
-				template
-			})?,
+			template: Template::new(template)?,
 			config,
 		};
 		changelog.process_commits();
@@ -40,32 +46,20 @@ impl<'a> Changelog<'a> {
 	/// criteria set by configuration file.
 	fn process_commits(&mut self) {
 		debug!("Processing the commits...");
-		let config = &self.config;
 		self.releases.iter_mut().for_each(|release| {
 			release.commits = release
 				.commits
 				.iter()
-				.filter_map(|commit| {
-					match commit.process(
-						config.git.commit_parsers.as_ref(),
-						config.git.filter_commits,
-						config.git.conventional_commits,
-					) {
-						Ok(commit) => Some(commit),
-						Err(e) => {
-							trace!(
-								"{} - {} ({})",
-								commit.id[..7].to_string(),
-								e,
-								commit
-									.message
-									.lines()
-									.next()
-									.unwrap_or_default()
-									.trim()
-							);
-							None
-						}
+				.filter_map(|commit| match commit.process(&self.config.git) {
+					Ok(commit) => Some(commit),
+					Err(e) => {
+						trace!(
+							"{} - {} ({})",
+							commit.id[..7].to_string(),
+							e,
+							commit.message.lines().next().unwrap_or_default().trim()
+						);
+						None
 					}
 				})
 				.collect::<Vec<Commit>>();
@@ -76,6 +70,7 @@ impl<'a> Changelog<'a> {
 	fn process_releases(&mut self) {
 		debug!("Processing the releases...");
 		let skip_regex = self.config.git.skip_tags.as_ref();
+		let mut skipped_tags = Vec::new();
 		self.releases = self
 			.releases
 			.clone()
@@ -92,6 +87,7 @@ impl<'a> Changelog<'a> {
 						.map(|r| {
 							let skip_tag = r.is_match(version);
 							if skip_tag {
+								skipped_tags.push(version.clone());
 								trace!("Skipping release: {}", version)
 							}
 							skip_tag
@@ -102,6 +98,25 @@ impl<'a> Changelog<'a> {
 				}
 			})
 			.collect();
+		for skipped_tag in &skipped_tags {
+			if let Some(release_index) = self.releases.iter().position(|release| {
+				release
+					.previous
+					.as_ref()
+					.and_then(|release| release.version.as_ref()) ==
+					Some(skipped_tag)
+			}) {
+				if let Some(previous_release) =
+					self.releases.get_mut(release_index + 1)
+				{
+					previous_release.previous = None;
+					self.releases[release_index].previous =
+						Some(Box::new(previous_release.clone()));
+				} else if release_index == self.releases.len() - 1 {
+					self.releases[release_index].previous = None;
+				}
+			}
+		}
 	}
 
 	/// Generates the changelog and writes it to the given output.
@@ -133,6 +148,13 @@ impl<'a> Changelog<'a> {
 		write!(out, "{}", changelog)?;
 		Ok(())
 	}
+
+	/// Prints the changelog context to the given output.
+	pub fn write_context<W: Write>(&self, out: &mut W) -> Result<()> {
+		let output = Releases(&self.releases).as_json()?;
+		writeln!(out, "{output}")?;
+		Ok(())
+	}
 }
 
 #[cfg(test)]
@@ -141,6 +163,7 @@ mod test {
 	use git_cliff_core::config::{
 		ChangelogConfig,
 		CommitParser,
+		CommitPreprocessor,
 		GitConfig,
 	};
 	use git_cliff_core::regex::Regex;
@@ -151,7 +174,7 @@ mod test {
 		let config = Config {
 			changelog: ChangelogConfig {
 				header: Some(String::from("# Changelog")),
-				body:   String::from(
+				body:   Some(String::from(
 					r#"{% if version %}
 				## Release [{{ version }}] - {{ timestamp | date(format="%Y-%m-%d") }}
 				({{ commit_id }}){% else %}
@@ -161,13 +184,18 @@ mod test {
 				#### {{ group }}{% for commit in commits %}
 				- {{ commit.message }}{% endfor %}
 				{% endfor %}{% endfor %}"#,
-				),
+				)),
 				footer: Some(String::from("------------")),
 				trim:   Some(true),
 			},
 			git:       GitConfig {
-				conventional_commits: true,
-				commit_parsers:       Some(vec![
+				conventional_commits:  Some(true),
+				filter_unconventional: Some(false),
+				commit_preprocessors:  Some(vec![CommitPreprocessor {
+					pattern: Regex::new("<preprocess>").unwrap(),
+					replace: String::from("this commit is preprocessed"),
+				}]),
+				commit_parsers:        Some(vec![
 					CommitParser {
 						message:       Regex::new("feat*").ok(),
 						body:          None,
@@ -182,17 +210,28 @@ mod test {
 						default_scope: None,
 						skip:          None,
 					},
+					CommitParser {
+						message:       Regex::new("merge*").ok(),
+						body:          None,
+						group:         None,
+						default_scope: None,
+						skip:          Some(true),
+					},
 					CommitParser {
 						message:       Regex::new(".*").ok(),
 						body:          None,
 						group:         Some(String::from("Other")),
-						default_scope: None,
+						default_scope: Some(String::from("other")),
 						skip:          None,
 					},
 				]),
-				filter_commits:       Some(false),
-				tag_pattern:          None,
-				skip_tags:            Regex::new("v3.*").ok(),
+				filter_commits:        Some(false),
+				tag_pattern:           None,
+				skip_tags:             Regex::new("v3.*").ok(),
+				ignore_tags:           None,
+				date_order:            Some(false),
+				sort_commits:          Some(String::from("oldest")),
+				link_parsers:          None,
 			},
 		};
 		let test_release = Release {
@@ -202,6 +241,10 @@ mod test {
 					String::from("0bc123"),
 					String::from("feat(app): add cool features"),
 				),
+				Commit::new(
+					String::from("000000"),
+					String::from("support unconventional commits"),
+				),
 				Commit::new(
 					String::from("0bc123"),
 					String::from("feat: support unscoped commits"),
@@ -218,6 +261,10 @@ mod test {
 					String::from("0jkl12"),
 					String::from("chore(app): do nothing"),
 				),
+				Commit::new(
+					String::from("qwerty"),
+					String::from("chore: <preprocess>"),
+				),
 			],
 			commit_id: Some(String::from("0bc123")),
 			timestamp: 50000000,
@@ -300,6 +347,10 @@ mod test {
 			#### app
 			- do nothing
 
+			#### other
+			- support unconventional commits
+			- this commit is preprocessed
+
 			#### ui
 			- make good stuff
 			------------"#
